@@ -21,6 +21,7 @@
 
 #include <QFile>
 #include <QTextStream>
+#include <KLocale>
 #include <util/file.h>
 #include <util/log.h>
 #include <diskio/chunkmanager.h>
@@ -39,31 +40,32 @@
 #include <interfaces/monitorinterface.h>
 #include <peer/packetwriter.h>
 #include <peer/accessmanager.h>
+#include <peer/chunkcounter.h>
 #include "chunkselector.h"
 #include "version.h"
 #include "webseed.h"
 
 
+
 namespace bt
 {
 
 	bool Downloader::use_webseeds = true;
 	
 	Downloader::Downloader(Torrent & tor,PeerManager & pman,ChunkManager & cman) 
-	: tor(tor),pman(pman),cman(cman),downloaded(0),tmon(0),chunk_selector(0),webseed_endgame_mode(false)
+	: tor(tor),pman(pman),cman(cman),bytes_downloaded(0),tmon(0),chunk_selector(0),webseed_endgame_mode(false)
 	{
 		webseeds_on = use_webseeds;
 		pman.setPieceHandler(this);
-		chunk_selector = new ChunkSelector(cman,*this,pman);
+		chunk_selector = new ChunkSelector();
+		chunk_selector->init(&cman,this,&pman);
 		
 		Uint64 total = tor.getTotalSize();
-		downloaded = (total - cman.bytesLeft());
+		bytes_downloaded = (total - cman.bytesLeft());
 		curr_chunks_downloaded = 0;
 		unnecessary_data = 0;
 	
 		current_chunks.setAutoDelete(true);
-		connect(&pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
-		connect(&pman,SIGNAL(peerKilled(Peer* )),this,SLOT(onPeerKilled(Peer*)));
 		
 		active_webseed_downloads = 0;
 		const KUrl::List & urls = tor.getWebSeeds();
@@ -110,9 +112,11 @@ namespace bt
 			delete chunk_selector;
 		
 		if (!csel) // check if a custom one was provided, if not create a default one
-			chunk_selector = new ChunkSelector(cman,*this,pman);
+			chunk_selector = new ChunkSelector();
 		else
 			chunk_selector = csel;
+		
+		chunk_selector->init(&cman,this,&pman);
 	}
 
 	
@@ -137,15 +141,15 @@ namespace bt
 				tmon->downloadRemoved(cd);
 				
 			if (ok)
-				downloaded += p.getLength();
+				bytes_downloaded += p.getLength();
 			
 			if (!finished(cd))
 			{
 				// if the chunk fails don't count the bytes downloaded
-				if (cd->getChunk()->getSize() > downloaded)
-					downloaded = 0;
+				if (cd->getChunk()->getSize() > bytes_downloaded)
+					bytes_downloaded = 0;
 				else
-					downloaded -= cd->getChunk()->getSize();
+					bytes_downloaded -= cd->getChunk()->getSize();
 				current_chunks.erase(p.getIndex());
 			}
 			else
@@ -161,7 +165,7 @@ namespace bt
 		else
 		{
 			if (ok)
-				downloaded += p.getLength();
+				bytes_downloaded += p.getLength();
 		}
 			
 		if (!ok)
@@ -384,7 +388,7 @@ namespace bt
 	}
 	
 
-	bool Downloader::areWeDownloading(Uint32 chunk) const
+	bool Downloader::downloading(Uint32 chunk) const
 	{
 		return current_chunks.find(chunk) != 0;
 	}
@@ -400,7 +404,7 @@ namespace bt
 				return false;
 		}
 		
-		return !areWeDownloading(chunk);
+		return !downloading(chunk);
 	}
 	
 	Uint32 Downloader::numDownloadersForChunk(Uint32 chunk) const
@@ -412,24 +416,19 @@ namespace bt
 		return cd->getNumDownloaders();
 	}
 	
-	void Downloader::onNewPeer(Peer* peer)
-	{		
-		PieceDownloader* pd = peer->getPeerDownloader();
-		piece_downloaders.append(pd);
+	void Downloader::addPieceDownloader(PieceDownloader* peer)
+	{
+		piece_downloaders.append(peer);
 	}
 
-	void Downloader::onPeerKilled(Peer* peer)
+	void Downloader::removePieceDownloader(PieceDownloader* peer)
 	{
-		PieceDownloader* pd = peer->getPeerDownloader();
-		if (pd)
+		for (CurChunkItr i = current_chunks.begin();i != current_chunks.end();++i)
 		{
-			for (CurChunkItr i = current_chunks.begin();i != current_chunks.end();++i)
-			{
-				ChunkDownload* cd = i->second;
-				cd->killed(pd);
-			}
-			piece_downloaders.removeAll(pd);
+			ChunkDownload* cd = i->second;
+			cd->killed(peer);
 		}
+		piece_downloaders.removeAll(peer);
 	}
 	
 	bool Downloader::finished(ChunkDownload* cd)
@@ -452,12 +451,7 @@ namespace bt
 				
 				cman.chunkDownloaded(c->getIndex());
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded " << endl;
-				// tell everybody we have the Chunk
-				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
-				{
-					pman.getPeer(i)->getPacketWriter().sendHave(c->getIndex());
-				}
-				
+				pman.sendHave(c->getIndex());
 				emit chunkDownloaded(c->getIndex());
 			}
 			catch (Error & e)
@@ -605,7 +599,7 @@ namespace bt
 			return;
 
 		// recalculate downloaded bytes
-		downloaded = (tor.getTotalSize() - cman.bytesLeft());
+		bytes_downloaded = (tor.getTotalSize() - cman.bytesLeft());
 
 		CurrentChunksHeader chdr;
 		fptr.read(&chdr,sizeof(CurrentChunksHeader));
@@ -653,7 +647,7 @@ namespace bt
 			else
 			{
 				current_chunks.insert(hdr.index,cd);
-				downloaded += cd->bytesDownloaded();
+				bytes_downloaded += cd->bytesDownloaded();
 		
 				if (tmon)
 					tmon->downloadStarted(cd);
@@ -760,32 +754,20 @@ namespace bt
 	void Downloader::recalcDownloaded()
 	{
 		Uint64 total = tor.getTotalSize();
-		downloaded = (total - cman.bytesLeft());
+		bytes_downloaded = (total - cman.bytesLeft());
 	}
 	
 	void Downloader::onChunkReady(Chunk* c)
 	{
-		PieceDataPtr piece = c->getPiece(0,c->getSize(),false);
-		
+		WebSeed* ws = webseeds_chunks.find(c->getIndex());
 		webseeds_chunks.erase(c->getIndex());
-		if (!piece)
-		{
-			// reset chunk but only when no other peer is downloading it
-			if (!current_chunks.find(c->getIndex()))
-				cman.resetChunk(c->getIndex());
-			
-			chunk_selector->reinsert(c->getIndex());
-			return;
-		}
-		
-
-		SHA1Hash h = SHA1Hash::generate(piece->data(),c->getSize());
-		if (tor.verifyHash(h,c->getIndex()))
+		PieceData::Ptr piece = c->getPiece(0,c->getSize(),true);
+		if (piece && c->checkHash(tor.getHash(c->getIndex())))
 		{
 			// hash ok so save it
 			try
 			{
-				downloaded += c->getSize();
+				bytes_downloaded += c->getSize();
 				
 				foreach (WebSeed* ws,webseeds)
 				{
@@ -799,6 +781,8 @@ namespace bt
 				{
 					// A ChunkDownload is ongoing for this chunk so kill it, we have the chunk
 					cd->cancelAll();
+					if (tmon)
+						tmon->downloadRemoved(cd);
 					current_chunks.erase(c->getIndex());
 				}
 				
@@ -807,10 +791,7 @@ namespace bt
 			
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded via webseed ! " << endl;
 				// tell everybody we have the Chunk
-				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
-				{
-					pman.getPeer(i)->getPacketWriter().sendHave(c->getIndex());
-				}
+				pman.sendHave(c->getIndex());
 			}
 			catch (Error & e)
 			{
@@ -821,14 +802,12 @@ namespace bt
 		else
 		{
 			Out(SYS_GEN|LOG_IMPORTANT) << "Hash verification error on chunk "  << c->getIndex() << endl;
-			Out(SYS_GEN|LOG_IMPORTANT) << "Is        : " << h << endl;
-			Out(SYS_GEN|LOG_IMPORTANT) << "Should be : " << tor.getHash(c->getIndex()) << endl;
-		
 			// reset chunk but only when no other peer is downloading it
 			if (!current_chunks.find(c->getIndex()))
 				cman.resetChunk(c->getIndex());
 			
 			chunk_selector->reinsert(c->getIndex());
+			ws->disable(i18n("Disabled because webseed does not match torrent"));
 		}
 	}
 	
@@ -976,7 +955,12 @@ namespace bt
 		}
 	}
 	
-	ChunkDownload* Downloader::getDownload(Uint32 chunk)
+	ChunkDownload* Downloader::download(Uint32 chunk)
+	{
+		return current_chunks.find(chunk);
+	}
+	
+	const bt::ChunkDownload* Downloader::download(Uint32 chunk) const
 	{
 		return current_chunks.find(chunk);
 	}
@@ -985,7 +969,6 @@ namespace bt
 	{
 		use_webseeds = on;
 	}
-
 }
 
 #include "downloader.moc"
